#pragma region License
/*
License (OLC-3)
~~~~~~~~~~~~~~~

Copyright 2018 - 2023 OneLoneCoder.com

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions or derivations of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions or derivative works in binary form must reproduce the above
copyright notice. This list of conditions and the following	disclaimer must be
reproduced in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may
be used to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS	"AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT	HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL,	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#pragma endregion
#pragma once
#include "olcPixelGameEngine.h"
#include "Key.h"

struct Player;

struct PrecastData{
	float castTime;
	float range;
	float size;
	//Whether or not this ability requires precasting (automatically set to true when precast time is greater than zero)
	bool precastTargetingRequired=false;
	PrecastData();
	//Cast an ability without a targeting indicator, just starts the cast.
	PrecastData(float castTime);
	PrecastData(float castTime,float range,float size);
};

//Abilities are tied to class data which is defined in Class.cpp.
struct Ability{
	std::string name="";
	std::string shortName="";
	std::string description="";
	float cooldown=0;
	float COOLDOWN_TIME=0;
	int manaCost=0;
	Pixel barColor1,barColor2;
	PrecastData precastInfo;
	bool canCancelCast=false;
	InputGroup*input;
	std::string icon;
	bool waitForRelease=false;
	//Ability action function, returns true if the ability can be casted, otherwise returns false.
	//	Argument 1: Player* - player pointer
	//  Argument 2: vf2d - The returned precast target position (if the ability needs to be aimed, otherwise {})
	std::function<bool(Player*,vf2d)>action=[](Player*,vf2d){return false;};
	static InputGroup DEFAULT;
	Ability();
	//NOTE: icon expects the actual name relative to the "Ability Icons" directory for this constructor!
	Ability(std::string name,std::string shortName,std::string description,float cooldownTime,int manaCost,InputGroup*input,std::string icon,Pixel barColor1=VERY_DARK_RED,Pixel barColor2=DARK_RED,PrecastData precastInfo={},bool canCancelCast=false);
};