#pragma region License
/*
License (OLC-3)
~~~~~~~~~~~~~~~

Copyright 2018 - 2023 OneLoneCoder.com

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions or derivations of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions or derivative works in binary form must reproduce the above
copyright notice. This list of conditions and the following	disclaimer must be
reproduced in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may
be used to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS	"AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT	HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL,	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#pragma endregion
#pragma once
#include <string>
#include "Player.h"

INCLUDE_DATA

struct ClassInfo{
	std::string className;
	std::string classFullImgName;
	int baseHealth;
	int baseAtk;
	float healthGrowthRate;
	float atkGrowthRate;
	Ability*ability1;
	Ability*ability2;
	Ability*ability3;
	Ability*rightClickAbility;
};

class classutils{
public:
	static inline Class StringToClass(std::string className){
		const std::vector<std::string>&classList=DATA["class_list"].GetValues();
		auto it=std::find(classList.begin(),classList.end(),className);
		int element=int(std::distance(classList.begin(),it));
		return Class(1<<element); //Yes...It's bitwise flags, who in god's name knows why I did this.
	};
	static inline ClassInfo GetClassInfo(std::string className){
		ClassInfo data{
			DATA.GetProperty(className+".ClassName").GetString(),
			DATA.GetProperty(className+".FullRender").GetString(),
			DATA.GetProperty(className+".BaseHealth").GetInt(),
			DATA.GetProperty(className+".BaseAtk").GetInt(),
			float(DATA.GetProperty(className+".HealthGrowthRate").GetReal()),
			float(DATA.GetProperty(className+".AtkGrowthRate").GetReal())
		};

		if(className=="Warrior"){
			data.rightClickAbility=&Warrior::rightClickAbility;
			data.ability1=&Warrior::ability1;
			data.ability2=&Warrior::ability2;
			data.ability3=&Warrior::ability3;
		}else
		if(className=="Wizard"){
			data.rightClickAbility=&Wizard::rightClickAbility;
			data.ability1=&Wizard::ability1;
			data.ability2=&Wizard::ability2;
			data.ability3=&Wizard::ability3;
		}else
		if(className=="Ranger"){
			data.rightClickAbility=&Ranger::rightClickAbility;
			data.ability1=&Ranger::ability1;
			data.ability2=&Ranger::ability2;
			data.ability3=&Ranger::ability3;
		}else
		if(className=="Trapper"){
			data.rightClickAbility=&Trapper::rightClickAbility;
			data.ability1=&Trapper::ability1;
			data.ability2=&Trapper::ability2;
			data.ability3=&Trapper::ability3;
		}else
		if(className=="Witch"){
			data.rightClickAbility=&Witch::rightClickAbility;
			data.ability1=&Witch::ability1;
			data.ability2=&Witch::ability2;
			data.ability3=&Witch::ability3;
		}else
		if(className=="Thief"){
			data.rightClickAbility=&Thief::rightClickAbility;
			data.ability1=&Thief::ability1;
			data.ability2=&Thief::ability2;
			data.ability3=&Thief::ability3;
		}else{
			ERR("WARNING! Could not get class info for non-existent class "+className+"!")
		}

		return data;
	}
};