#pragma region License
/*
License (OLC-3)
~~~~~~~~~~~~~~~

Copyright 2018 - 2023 OneLoneCoder.com

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions or derivations of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions or derivative works in binary form must reproduce the above
copyright notice. This list of conditions and the following	disclaimer must be
reproduced in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may
be used to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS	"AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT	HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL,	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#pragma endregion
#include "Crawler.h"
#include "olcPixelGameEngine.h"
#include "safemap.h"
#include "Item.h"
#include "MenuItemButton.h"
#include "MenuLabel.h"
#include "InventoryScrollableWindowComponent.h"

typedef Attribute A;

void Menu::InitializeConsumableInventoryWindow(){
	int invWidth="ThemeGlobal.InventoryWidth"_I;
	int initialInvHeight="ThemeGlobal.InventoryHeight"_I;

	int itemSpacing="ThemeGlobal.InventoryItemSpacing"_I;
	int buttonSize="ThemeGlobal.InventoryButtonSize"_I;
	int totalSpacing=buttonSize+itemSpacing;

	vf2d windowSize={float(totalSpacing*invWidth-itemSpacing+2+24),float(totalSpacing*(3+1)-itemSpacing+64)}; //Need space for the button.

	Menu*inventoryWindow=CreateMenu(INVENTORY,CENTERED,windowSize);

	inventoryWindow->I(A::LOADOUT_SLOT)=0;

	InventoryScrollableWindowComponent*inventory=NEW InventoryScrollableWindowComponent(INVENTORY,{{1,20},{windowSize.x,float(totalSpacing*3-itemSpacing)}},"Consumables","itemName","itemDescription",
		[&](MenuFuncData data){
			MenuItemButton*button=(MenuItemButton*)data.component;
			data.game->ClearLoadoutItem(data.menu.I(A::LOADOUT_SLOT));
			for(MenuComponent*component:data.parentComponent->GetComponents()){ //HACK ALERT! If we are accessing a parent component, it's because we are dealing with a scrolling window component, which has sub-components. So this should be a safe cast to make.
				if(component->GetName().starts_with("item")){
					MenuItemButton*button2=(MenuItemButton*)component;//HACK ALERT! This is probably an item since we populated item lists using this name for the components. So we assume these are MenuItemButton classes.
					if(button2==button){
						if(button2->selected!=-1){
							data.game->ClearLoadoutItem(button2->selected);
						}
						button2->selected=-1;
					}
					if(button2->selected==data.menu.I(A::LOADOUT_SLOT)){
						button2->selected=-1;
					}
				}
			}
			button->selected=data.menu.I(A::LOADOUT_SLOT);
			data.game->SetLoadoutItem(button->selected,button->GetItem().Name());
			return true;
		});
	inventoryWindow->AddComponent("inventory",inventory);

	//We don't have to actually populate the inventory list because now when an item gets added, it will automatically add the correct component in for us.

	MenuLabel*inventoryTypeLabel=NEW MenuLabel(INVENTORY,{{0,0},{windowSize.x-1,18}},"Consumables",2,ComponentAttr::SHADOW|ComponentAttr::BACKGROUND|ComponentAttr::OUTLINE);
	inventoryWindow->AddComponent("Inventory Type Label",inventoryTypeLabel);

	MenuLabel*itemNameLabel=NEW MenuLabel{INVENTORY,geom2d::rect<float>(vf2d{2,float(initialInvHeight*totalSpacing+itemSpacing-16)},{windowSize.x-4,windowSize.y-108}),"",1,ComponentAttr::SHADOW};
	inventoryWindow->AddComponent("itemName",itemNameLabel);
	float itemDescriptionLabelY=float(initialInvHeight*totalSpacing+itemSpacing);
	MenuLabel*itemDescriptionLabel=NEW MenuLabel{INVENTORY,geom2d::rect<float>(vf2d{2,itemDescriptionLabelY},{windowSize.x-4,windowSize.y-108}),"",1,ComponentAttr::SHADOW};
	inventoryWindow->AddComponent("itemDescription",itemDescriptionLabel);

	MenuComponent*okButton=NEW MenuComponent(INVENTORY,{{windowSize.x/2-24,itemDescriptionLabelY+56},{48,12}},"Ok",[](MenuFuncData data){Menu::CloseMenu();return true;});

	inventoryWindow->AddComponent("OK Button",okButton);
}