#pragma region License
/*
License (OLC-3)
~~~~~~~~~~~~~~~

Copyright 2018 - 2023 OneLoneCoder.com

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions or derivations of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions or derivative works in binary form must reproduce the above
copyright notice. This list of conditions and the following	disclaimer must be
reproduced in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may
be used to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS	"AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT	HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL,	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#pragma endregion
#pragma once
#include <string>
#include <functional>
#include <map>
#include "olcPixelGameEngine.h"
#include "olcUTIL_DataFile.h"

class Crawler;
class ItemInfo;
class ItemProps;

typedef std::string IT;
typedef std::string ITCategory;

typedef std::function<bool(Crawler*,ItemProps)> ItemScript;

class Item{
	friend class Inventory;
	friend class Crawler;
private:
	//The amount in the current item stack.
	uint32_t amt;
	ItemInfo*it;
public:
	Item();
	Item(uint32_t amt,IT item);
	uint32_t Amt();
	std::string Name();
	std::string Description();
	ITCategory Category();
	Decal*Decal();
	ItemScript&OnUseAction();
	float CastTime();
	float CooldownTime();
	bool IsBlank();
	static Item BLANK;
	bool operator==(const Item&rhs)const;
};

class Inventory{
	friend class ItemInfo;
	friend class Item;
public:
	static void AddItem(IT it,uint32_t amt=1,bool monsterDrop=false);
	//Returns the actual amount available in your main inventory.
	static uint32_t GetItemCount(IT it);
	static Item GetItem(IT it);
	//Auto-executes its use function and removes the amt specified from the inventory. Multiple amounts will cause the item to execute its useFunc multiple times.
	static bool UseItem(IT it,uint32_t amt=1);
	static bool RemoveItem(IT it,ITCategory inventory,uint32_t amt = 1);
	static bool RemoveItem(IT it,uint32_t amt=1);
	static std::vector<Item>&get(ITCategory itemCategory);
	static void Clear(ITCategory itemCategory);

	static bool SwapItems(ITCategory itemCategory,uint32_t slot1,uint32_t slot2);
	//Makes sure this is a valid category. Will error out if it doesn't exist! Use for ERROR HANDLING!
	static bool ValidateItemCategory(ITCategory itemCategory){
		get(itemCategory);
		return true;
	}
private:
	static void InsertIntoSortedInv(IT item);
	static void InsertIntoStageInventoryCategory(IT item,uint32_t amt,bool monsterDrop);
	static bool ExecuteAction(IT item);
	static std::map<IT,Item>_inventory;
	//Only contains "1" of every item, as this is a map to index items and not the actual storage of items!
	static std::map<ITCategory,std::vector<Item>>sortedInv;
};

class ItemProps{
	friend class ItemInfo;
	utils::datafile*scriptProps; 
	utils::datafile*customProps; 
public:
	ItemProps(utils::datafile*scriptProps,utils::datafile*customProps);
	int GetIntProp(std::string prop);
	float GetFloatProp(std::string prop);
	std::string GetStringProp(std::string prop);
};

class ItemInfo{
	friend class Inventory;
	std::string name;
	std::string description;
	std::string category;
	float castTime=0;
	float cooldownTime=0;
	Decal*img;
	//Returns true if the item can be used, false otherwise
	std::string useFunc="";
	//Custom properties for this specific item's script.
	static utils::datafile NOPROPS;
	ItemProps customProps;

private:
	static void InitializeScripts();
public:
	static void InitializeItems();
	ItemInfo();
	std::string Name();
	std::string Description();
	ITCategory Category();
	Decal*Decal();
	/*
	For the useFunc, return true if the item can be used, false otherwise.
	*/
	ItemScript&OnUseAction();
	float CastTime();
	float CooldownTime();
};

class ItemOverlay{
	ItemInfo it;
	float timer=0;
	float xOffset=0;
	float width=0; //How wide the entire label is.
	static std::vector<ItemOverlay>items;
	ItemOverlay(ItemInfo item);
public:
	static void AddToItemOverlay(const ItemInfo&it);
	static void Update();
	static void Draw();
	void ResetTimer();
};