#pragma region License
/*
License (OLC-3)
~~~~~~~~~~~~~~~

Copyright 2018 - 2023 OneLoneCoder.com

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions or derivations of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions or derivative works in binary form must reproduce the above
copyright notice. This list of conditions and the following	disclaimer must be
reproduced in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may
be used to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS	"AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT	HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL,	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#pragma endregion
#pragma once
#include "Menu.h"
#include "MenuLabel.h"
#include "MenuIconButton.h"
#include "DEFINES.h"
#include "Crawler.h"
#include "Item.h"
#include "safemap.h"
#include "olcPGEX_Graphics2D.h"
#include "drawutil.h"

INCLUDE_game
INCLUDE_ITEM_DATA

class MenuItemButton:public MenuIconButton{
private:
    std::vector<Item>&invRef;
    int inventoryIndex=0;
    MenuType itemDescriptionMenu;
    std::string itemNameLabelName;
    std::string itemDescriptionLabelName;
public:
    int selected=-1; //0-2 representing which loadout slot this item consumes. -1 means not selected.
    inline MenuItemButton(MenuType parent,geom2d::rect<float>rect,std::vector<Item>&invRef,int invIndex,MenuFunc onClick,MenuType itemDescriptionMenu,std::string itemNameLabelName,std::string itemDescriptionLabelName,IconButtonAttr attributes=IconButtonAttr::SELECTABLE)
        :MenuIconButton(parent,rect,invRef.size()>invIndex?invRef[invIndex].Decal():nullptr,onClick,attributes),invRef(invRef),inventoryIndex(invIndex),itemDescriptionMenu(itemDescriptionMenu),itemNameLabelName(itemNameLabelName),itemDescriptionLabelName(itemDescriptionLabelName){
        draggable=false;
        valid=invRef.size()>invIndex;
    }
    inline Item GetItem(){
        return Inventory::GetItem(invRef.at(inventoryIndex).Name());
    }
    //Returns true if the item has been consumed completely and there are 0 remaining of that type in our inventory.
    inline bool UseItem(uint32_t amt=1){
        if(invRef.size()<=inventoryIndex)return false;
        return Inventory::UseItem(invRef.at(inventoryIndex).Name(),amt);
    }
protected:
    virtual inline void Update(Crawler*game)override{
	    MenuIconButton::Update(game);
        valid=invRef.size()>inventoryIndex&&ITEM_DATA.count(invRef[inventoryIndex].Name());
        if(valid){
            icon=invRef[inventoryIndex].Decal();
            if(hovered){
                Component<MenuLabel>(itemDescriptionMenu,itemNameLabelName)->label=invRef[inventoryIndex].Name();
                Component<MenuLabel>(itemDescriptionMenu,itemDescriptionLabelName)->label=invRef[inventoryIndex].Description();
            }
        }else{
            icon=nullptr;
            if(hovered){
                Component<MenuLabel>(itemDescriptionMenu,itemNameLabelName)->label="";
                Component<MenuLabel>(itemDescriptionMenu,itemDescriptionLabelName)->label="";
            }
        }
    }
    virtual inline void Draw(Crawler*game,vf2d parentPos)override{
        MenuIconButton::Draw(game,parentPos);
        if(selected!=-1){
		    drawutil::DrawCrosshair(game,{parentPos+rect.pos,rect.size},0);
        }
    }
    virtual inline void DrawDecal(Crawler*game,vf2d parentPos,bool focused)override{
	    MenuIconButton::DrawDecal(game,parentPos,focused);
        if(valid){
            int itemQuantity=Inventory::GetItemCount(invRef.at(inventoryIndex).Name()); //Normally we'd retrieve how many of this item we have from our inventory...However Monster Loot and Stage Loot inventories are special and hold their own inventory counts...
            if(&invRef==&Inventory::get("Monster Loot")||&invRef==&Inventory::get("Stage Loot")){
                itemQuantity=invRef.at(inventoryIndex).Amt(); //So the item quantity comes from the stack itself and not our main inventory.
            }
            std::string quantityText="x"+std::to_string(itemQuantity);
            vf2d textSize=vf2d(game->GetTextSizeProp(quantityText))*0.5;
            vf2d drawPos=parentPos+rect.pos+rect.size-textSize;
            if(PointWithinParent(this,drawPos)){
                game->DrawShadowStringDecal(drawPos,quantityText,WHITE,BLACK,{0.5,0.5},0.5);
            }
        }
    }
    inline MenuComponent*PickUpDraggableItem()override final{
        if(valid){
            MenuItemButton*pickUp=NEW MenuItemButton(parentMenu,rect,invRef,inventoryIndex,onClick,itemDescriptionMenu,itemNameLabelName,itemDescriptionLabelName);
            valid=false;
            return pickUp;
        }else{
            return nullptr;
        }
    }

    inline bool DropDraggableItem(MenuComponent*draggable)override final{
        //HACK Warning! We're making a bold assumption that every component that is draggable is of the same type! This may change in the future....
        MenuItemButton*draggedItem=(MenuItemButton*)draggable;
        ITCategory cat=draggedItem->invRef.at(draggedItem->inventoryIndex).Category();
        return Inventory::SwapItems(cat,draggedItem->inventoryIndex,inventoryIndex);
    }
};
