#pragma region License
/*
License (OLC-3)
~~~~~~~~~~~~~~~

Copyright 2018 - 2023 OneLoneCoder.com

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions or derivations of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions or derivative works in binary form must reproduce the above
copyright notice. This list of conditions and the following	disclaimer must be
reproduced in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may
be used to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS	"AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT	HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL,	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#pragma endregion
#pragma once
#include "Menu.h"
#include "MenuLabel.h"
#include "MenuItemButton.h"
#include "DEFINES.h"
#include "Crawler.h"
#include "Item.h"
#include "safemap.h"
#include "olcPGEX_Graphics2D.h"
#include "drawutil.h"

INCLUDE_game
INCLUDE_ITEM_DATA

class MenuItemItemButton:public MenuIconButton{
private:
    Item&itemRef;
    MenuType itemDescriptionMenu;
    std::string itemNameLabelName;
    std::string itemDescriptionLabelName;
public:
    inline MenuItemItemButton(MenuType parent,geom2d::rect<float>rect,Item&itemRef,MenuType menuDest,MenuFunc onClick,MenuType itemDescriptionMenu,std::string itemNameLabelName,std::string itemDescriptionLabelName)
        :MenuIconButton(parent,rect,(!itemRef.IsBlank())?itemRef.Decal():nullptr,menuDest,onClick),itemRef(itemRef),itemDescriptionMenu(itemDescriptionMenu),itemNameLabelName(itemNameLabelName),itemDescriptionLabelName(itemDescriptionLabelName){
        draggable=false;
        valid=!itemRef.IsBlank();
    }
protected:
    virtual inline void Update(Crawler*game)override{
	    MenuIconButton::Update(game);
        valid=!itemRef.IsBlank();
        if(valid){
            icon=itemRef.Decal();
            if(hovered){
                Component<MenuLabel>(itemDescriptionMenu,itemNameLabelName)->label=itemRef.Name();
                Component<MenuLabel>(itemDescriptionMenu,itemDescriptionLabelName)->label=itemRef.Description();
            }
        }else{
            icon=nullptr;
            if(hovered){
                Component<MenuLabel>(itemDescriptionMenu,itemNameLabelName)->label="";
                Component<MenuLabel>(itemDescriptionMenu,itemDescriptionLabelName)->label="";
            }
        }
    }
    virtual inline void Draw(Crawler*game,vf2d parentPos)override{
        MenuIconButton::Draw(game,parentPos);
    }
    virtual inline void DrawDecal(Crawler*game,vf2d parentPos,bool focused)override{
	    MenuIconButton::DrawDecal(game,parentPos,focused);
        if(valid){
            std::string quantityText="x"+std::to_string(itemRef.Amt());
            vf2d textSize=vf2d(game->GetTextSizeProp(quantityText))*0.5;
            vf2d drawPos=parentPos+rect.pos+rect.size-textSize;
            if(PointWithinParent(this,drawPos)){
                game->DrawShadowStringDecal(drawPos,quantityText,WHITE,BLACK,{0.5,0.5},0.5);
            }
        }
    }
};