#pragma region License
/*
License (OLC-3)
~~~~~~~~~~~~~~~

Copyright 2018 - 2023 OneLoneCoder.com

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions or derivations of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions or derivative works in binary form must reproduce the above
copyright notice. This list of conditions and the following	disclaimer must be
reproduced in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may
be used to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS	"AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT	HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL,	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#pragma endregion
#include "Monster.h"
#include "DEFINES.h"
#include "Crawler.h"
#include "MonsterStrategyHelpers.h"

INCLUDE_game
INCLUDE_MONSTER_DATA

void Monster::STRATEGY::RUN_TOWARDS(Monster&m,float fElapsedTime,int strategyNumber){
	m.targetAcquireTimer=std::max(0.f,m.targetAcquireTimer-fElapsedTime);
	if(m.targetAcquireTimer==0){
		m.targetAcquireTimer=ConfigFloat("WaitTime");

		auto desiredTargetLine = geom2d::line(m.pos,game->GetPlayer()->GetPos());
		if(desiredTargetLine.length()>=ConfigInt("MaxDistance")/100.f*24){
			//Trim to max distance desired.
			m.target=desiredTargetLine.rpoint(ConfigInt("MaxDistance")/100.f*24);
		} else {
			m.target=desiredTargetLine.upoint(1.2f);
		}
		m.SetState(State::MOVE_TOWARDS);
		m.hasHitPlayer=false;
	}
	switch(m.state){
		case State::MOVE_TOWARDS:{
			if(geom2d::line(m.pos,m.target).length()>100*fElapsedTime*m.GetMoveSpdMult()){
				vf2d newPos=m.pos+geom2d::line(m.pos,m.target).vector().norm()*100*fElapsedTime*m.GetMoveSpdMult();
				if(!m.SetX(newPos.x)||!m.SetY(newPos.y)){
					m.StartPathfinding(4);
				}
				m.PerformJumpAnimation();
			} else {
				m.SetState(State::NORMAL);//Revert state once we've finished moving towards target.
				m.UpdateAnimation(MONSTER_DATA[m.id].GetIdleAnimation());
			}
		}break;
		case State::PATH_AROUND:{
			m.PathAroundBehavior(fElapsedTime);
		}break;
		default:{
		}
	}
}