#pragma region License
/*
License (OLC-3)
~~~~~~~~~~~~~~~

Copyright 2018 - 2023 OneLoneCoder.com

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions or derivations of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions or derivative works in binary form must reproduce the above
copyright notice. This list of conditions and the following	disclaimer must be
reproduced in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may
be used to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS	"AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT	HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL,	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#pragma endregion
#pragma once
#include "MenuLabel.h"
#include "DEFINES.h"
#include "Crawler.h"
#include "util.h"

INCLUDE_game
INCLUDE_MONSTER_DATA
INCLUDE_ANIMATION_DATA

class SpawnEncounterLabel:public MenuLabel{
    int monsterID;
    Animate2D::Animation<std::string>anim;
    Animate2D::AnimationState state;
public:
    inline SpawnEncounterLabel(MenuType parent,geom2d::rect<float>rect,std::string label,int monsterID)
        :MenuLabel(parent,rect,label),monsterID(monsterID){
        anim.AddState("IDLE",ANIMATION_DATA.at(MONSTER_DATA.at(monsterID).GetIdleAnimation()));
        anim.ChangeState(state,"IDLE");
        anim.UpdateState(state,util::random(1));
    }
    inline ~SpawnEncounterLabel(){
        
    }
protected:
    virtual void inline Update(Crawler*game)override{
	    MenuComponent::Update(game);
        anim.UpdateState(state,game->GetElapsedTime());
    }
    virtual void inline Draw(Crawler*game,vf2d parentPos)override{
        
    }
    virtual void inline DrawDecal(Crawler*game,vf2d parentPos,bool focused)override{
        const geom2d::rect<int>&imgRect=anim.GetFrame(state).GetSourceRect();
        vf2d imgSize=vf2d{rect.size.y,rect.size.y};
        vf2d imgScale=imgSize/vf2d{imgRect.size};
        game->DrawPartialDecal(parentPos+rect.pos,imgSize,anim.GetFrame(state).GetSourceImage()->Decal(),imgRect.pos,imgRect.size);
        float verticalAlignYOffset=(rect.size.y-8)/2;
        std::string monsterName=MONSTER_DATA.at(monsterID).GetDisplayName();
        vf2d monsterNameTextSize=game->GetTextSizeProp(monsterName);
        float textXSpaceAvailable=rect.size.x-imgSize.x-4-16/*12 for the scrollbar*/;
        float textXScaling=textXSpaceAvailable/monsterNameTextSize.x;
        game->DrawShadowStringPropDecal(parentPos+rect.pos+vf2d{imgSize.x+4,verticalAlignYOffset},monsterName,WHITE,BLACK,{std::min(1.f,textXScaling),1});
    }
};