#pragma region License
/*
License (OLC-3)
~~~~~~~~~~~~~~~

Copyright 2018 - 2023 OneLoneCoder.com

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions or derivations of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions or derivative works in binary form must reproduce the above
copyright notice. This list of conditions and the following	disclaimer must be
reproduced in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may
be used to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS	"AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT	HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL,	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#pragma endregion
#include "drawutil.h"
#include "Crawler.h"
#include "DEFINES.h"

INCLUDE_game

void drawutil::DrawCrosshair(Crawler*game,geom2d::rect<float>rect,float accTime,float pulsatingAmt,float borderThickness){
	vf2d crosshairExtension={std::min(0.25f*rect.size.x-borderThickness/2,12.f),std::min(0.25f*rect.size.y-borderThickness/2,12.f)};
	vf2d pulseAmt=vf2d{1,1}*std::abs(std::sin(accTime*3))*2;
	//Lower-Left Corner
	vf2d pos=rect.pos+vf2d{0,rect.size.y-borderThickness}+vf2d{-pulseAmt.x,pulseAmt.y};
	vf2d size={borderThickness+crosshairExtension.x,borderThickness};
	game->FillRect(pos,size,RED);
	pos=vf2d{pos.x,pos.y-crosshairExtension.y};
	size={borderThickness,crosshairExtension.y};
	game->FillRect(pos,size,RED);
	//Lower-Right Corner
	pos=rect.pos+vf2d{rect.size.x-borderThickness-crosshairExtension.x,rect.size.y-borderThickness}+vf2d{pulseAmt.x,pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->FillRect(pos,size,RED);
	pos=vf2d{pos.x+crosshairExtension.x,pos.y-crosshairExtension.y};
	size={borderThickness,crosshairExtension.y};
	game->FillRect(pos,size,RED);
	//Upper-Left Corner
	pos=rect.pos+vf2d{0,0}+vf2d{-pulseAmt.x,-pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->FillRect(pos,size,RED);
	pos=vf2d{pos.x,pos.y+borderThickness};
	size={borderThickness,crosshairExtension.y};
	game->FillRect(pos,size,RED);
	//Upper-Right Corner
	pos=rect.pos+vf2d{rect.size.x-borderThickness-crosshairExtension.x,0}+vf2d{pulseAmt.x,-pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->FillRect(pos,size,RED);
	pos=vf2d{pos.x+crosshairExtension.x,pos.y+borderThickness};
	size={borderThickness,crosshairExtension.y};
	game->FillRect(pos,size,RED);
};

void drawutil::DrawCrosshairTransformedView(olc::TransformedView&view,geom2d::rect<float>rect,float accTime,float pulsatingAmt,float borderThickness){
	vf2d crosshairExtension={std::min(0.25f*rect.size.x-borderThickness/2,12.f),std::min(0.25f*rect.size.y-borderThickness/2,12.f)};
	vf2d pulseAmt=vf2d{1,1}*std::abs(std::sin(accTime*3))*2;
	//Lower-Left Corner
	vf2d pos=rect.pos+vf2d{0,rect.size.y-borderThickness}+vf2d{-pulseAmt.x,pulseAmt.y};
	vf2d size={borderThickness+crosshairExtension.x,borderThickness};
	game->view.FillRect(pos,size,RED);
	pos=vf2d{pos.x,pos.y-crosshairExtension.y};
	size={borderThickness,crosshairExtension.y};
	game->view.FillRect(pos,size,RED);
	//Lower-Right Corner
	pos=rect.pos+vf2d{rect.size.x-borderThickness-crosshairExtension.x,rect.size.y-borderThickness}+vf2d{pulseAmt.x,pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->view.FillRect(pos,size,RED);
	pos=vf2d{pos.x+crosshairExtension.x,pos.y-crosshairExtension.y};
	size={borderThickness,crosshairExtension.y};
	game->view.FillRect(pos,size,RED);
	//Upper-Left Corner
	pos=rect.pos+vf2d{0,0}+vf2d{-pulseAmt.x,-pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->view.FillRect(pos,size,RED);
	pos=vf2d{pos.x,pos.y+borderThickness};
	size={borderThickness,crosshairExtension.y};
	game->view.FillRect(pos,size,RED);
	//Upper-Right Corner
	pos=rect.pos+vf2d{rect.size.x-borderThickness-crosshairExtension.x,0}+vf2d{pulseAmt.x,-pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->view.FillRect(pos,size,RED);
	pos=vf2d{pos.x+crosshairExtension.x,pos.y+borderThickness};
	size={borderThickness,crosshairExtension.y};
	game->view.FillRect(pos,size,RED);
}

void drawutil::DrawCrosshairDecal(Crawler*game,geom2d::rect<float>rect,float accTime,float pulsatingAmt,float borderThickness){
	vf2d crosshairExtension={std::min(0.25f*rect.size.x-borderThickness/2,12.f),std::min(0.25f*rect.size.y-borderThickness/2,12.f)};
	vf2d pulseAmt=vf2d{1,1}*std::abs(std::sin(accTime*3))*2;
	//Lower-Left Corner
	vf2d pos=rect.pos+vf2d{0,rect.size.y-borderThickness}+vf2d{-pulseAmt.x,pulseAmt.y};
	vf2d size={borderThickness+crosshairExtension.x,borderThickness};
	game->FillRectDecal(pos,size,RED);
	pos=vf2d{pos.x,pos.y-crosshairExtension.y};
	size={borderThickness,crosshairExtension.y};
	game->FillRectDecal(pos,size,RED);
	//Lower-Right Corner
	pos=rect.pos+vf2d{rect.size.x-borderThickness-crosshairExtension.x,rect.size.y-borderThickness}+vf2d{pulseAmt.x,pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->FillRectDecal(pos,size,RED);
	pos=vf2d{pos.x+crosshairExtension.x,pos.y-crosshairExtension.y};
	size={borderThickness,crosshairExtension.y};
	game->FillRectDecal(pos,size,RED);
	//Upper-Left Corner
	pos=rect.pos+vf2d{0,0}+vf2d{-pulseAmt.x,-pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->FillRectDecal(pos,size,RED);
	pos=vf2d{pos.x,pos.y+borderThickness};
	size={borderThickness,crosshairExtension.y};
	game->FillRectDecal(pos,size,RED);
	//Upper-Right Corner
	pos=rect.pos+vf2d{rect.size.x-borderThickness-crosshairExtension.x,0}+vf2d{pulseAmt.x,-pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->FillRectDecal(pos,size,RED);
	pos=vf2d{pos.x+crosshairExtension.x,pos.y+borderThickness};
	size={borderThickness,crosshairExtension.y};
	game->FillRectDecal(pos,size,RED);
};

void drawutil::DrawCrosshairDecalTransformedView(olc::TransformedView&view,geom2d::rect<float>rect,float accTime,float pulsatingAmt,float borderThickness){
	vf2d crosshairExtension={std::min(0.25f*rect.size.x-borderThickness/2,12.f),std::min(0.25f*rect.size.y-borderThickness/2,12.f)};
	vf2d pulseAmt=vf2d{1,1}*std::abs(std::sin(accTime*3))*2;
	//Lower-Left Corner
	vf2d pos=rect.pos+vf2d{0,rect.size.y-borderThickness}+vf2d{-pulseAmt.x,pulseAmt.y};
	vf2d size={borderThickness+crosshairExtension.x,borderThickness};
	game->view.FillRectDecal(pos,size,RED);
	pos=vf2d{pos.x,pos.y-crosshairExtension.y};
	size={borderThickness,crosshairExtension.y};
	game->view.FillRectDecal(pos,size,RED);
	//Lower-Right Corner
	pos=rect.pos+vf2d{rect.size.x-borderThickness-crosshairExtension.x,rect.size.y-borderThickness}+vf2d{pulseAmt.x,pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->view.FillRectDecal(pos,size,RED);
	pos=vf2d{pos.x+crosshairExtension.x,pos.y-crosshairExtension.y};
	size={borderThickness,crosshairExtension.y};
	game->view.FillRectDecal(pos,size,RED);
	//Upper-Left Corner
	pos=rect.pos+vf2d{0,0}+vf2d{-pulseAmt.x,-pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->view.FillRectDecal(pos,size,RED);
	pos=vf2d{pos.x,pos.y+borderThickness};
	size={borderThickness,crosshairExtension.y};
	game->view.FillRectDecal(pos,size,RED);
	//Upper-Right Corner
	pos=rect.pos+vf2d{rect.size.x-borderThickness-crosshairExtension.x,0}+vf2d{pulseAmt.x,-pulseAmt.y};
	size={borderThickness+crosshairExtension.x,borderThickness};
	game->view.FillRectDecal(pos,size,RED);
	pos=vf2d{pos.x+crosshairExtension.x,pos.y+borderThickness};
	size={borderThickness,crosshairExtension.y};
	game->view.FillRectDecal(pos,size,RED);
}